@@ -1,5 +1,5 @@
 # This file is part of the MapProxy project.
-# Copyright (C) 2010,2012 Omniscale <http://omniscale.de>
+# Copyright (C) 2010-2016 Omniscale <http://omniscale.de>
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,9 +16,9 @@
 """
 Image and tile manipulation (transforming, merging, etc).
 """
-from __future__ import with_statement
-
-from mapproxy.compat.image import Image, ImageColor, ImageChops
+import json
+from collections import namedtuple
+from mapproxy.compat.image import Image, ImageColor, ImageChops, ImageMath
 from mapproxy.compat.image import has_alpha_composite_support
 from mapproxy.image import BlankImageSource, ImageSource
 from mapproxy.image.opts import create_image, ImageOptions
@@ -35,12 +35,15 @@ class LayerMerger(object):
         self.layers = []
         self.cacheable = True
 
-    def add(self, layer_img, layer=None):
+    def add(self, img, coverage=None):
         """
         Add one layer image to merge. Bottom-layers first.
         """
-        if layer_img is not None:
-            self.layers.append((layer_img, layer))
+        if img is not None:
+            self.layers.append((img, coverage))
+
+
+class LayerMerger(LayerMerger):
 
     def merge(self, image_opts, size=None, bbox=None, bbox_srs=None, coverage=None):
         """
@@ -53,11 +56,11 @@ class LayerMerger(object):
         if not self.layers:
             return BlankImageSource(size=size, image_opts=image_opts, cacheable=True)
         if len(self.layers) == 1:
-            layer_img, layer = self.layers[0]
+            layer_img, layer_coverage = self.layers[0]
             layer_opts = layer_img.image_opts
             if (((layer_opts and not layer_opts.transparent) or image_opts.transparent)
                 and (not size or size == layer_img.size)
-                and (not layer or not layer.coverage or not layer.coverage.clip)
+                and (not layer_coverage or not layer_coverage.clip)
                 and not coverage):
                 # layer is opaque, no need to make transparent or add bgcolor
                 return layer_img
@@ -67,7 +70,7 @@ class LayerMerger(object):
 
         cacheable = self.cacheable
         result = create_image(size, image_opts)
-        for layer_img, layer in self.layers:
+        for layer_img, layer_coverage in self.layers:
             if not layer_img.cacheable:
                 cacheable = False
             img = layer_img.as_image()
@@ -77,14 +80,19 @@ class LayerMerger(object):
             else:
                 opacity = layer_image_opts.opacity
 
-            if layer and layer.coverage and layer.coverage.clip:
-                img = mask_image(img, bbox, bbox_srs, layer.coverage)
+            if layer_coverage and layer_coverage.clip:
+                img = mask_image(img, bbox, bbox_srs, layer_coverage)
 
             if result.mode != 'RGBA':
                 merge_composite = False
             else:
                 merge_composite = has_alpha_composite_support()
 
+            if 'transparency' in img.info:
+                # non-paletted PNGs can have a fixed transparency value
+                # convert to RGBA to have full alpha
+                img = img.convert('RGBA')
+
             if merge_composite:
                 if opacity is not None and opacity < 1.0:
                     # fade-out img to add opacity value
@@ -92,21 +100,21 @@ class LayerMerger(object):
                     alpha = img.split()[3]
                     alpha = ImageChops.multiply(
                         alpha,
-                        ImageChops.constant(alpha, 255 * opacity)
+                        ImageChops.constant(alpha, int(255 * opacity))
                     )
                     img.putalpha(alpha)
-                if img.mode == 'RGB':
-                    result.paste(img, (0, 0))
-                else:
+                if img.mode in ('RGBA', 'P'):
                     # assume paletted images have transparency
                     if img.mode == 'P':
                         img = img.convert('RGBA')
                     result = Image.alpha_composite(result, img)
+                else:
+                    result.paste(img, (0, 0))
             else:
                 if opacity is not None and opacity < 1.0:
                     img = img.convert(result.mode)
                     result = Image.blend(result, img, layer_image_opts.opacity)
-                elif img.mode == 'RGBA' or img.mode == 'P':
+                elif img.mode in ('RGBA', 'P'):
                     # assume paletted images have transparency
                     if img.mode == 'P':
                         img = img.convert('RGBA')
@@ -124,7 +132,103 @@ class LayerMerger(object):
 
         return ImageSource(result, size=size, image_opts=image_opts, cacheable=cacheable)
 
-def merge_images(images, image_opts, size=None):
+
+band_ops = namedtuple("band_ops", ["dst_band", "src_img", "src_band", "factor"])
+
+class BandMerger(object):
+    """
+    Merge bands from multiple sources into one image.
+
+       sources:
+           r: [{source: nir_cache, band: 0, factor: 0.4}, {source: dop_cache, band: 0, factor: 0.6}]
+           g: [{source: dop_cache, band: 2}]
+           b: [{source: dop_cache, band: 1}]
+
+       sources:
+           l: [
+               {source: dop_cache, band: 0, factor: 0.6},
+               {source: dop_cache, band: 1, factor: 0.3},
+               {source: dop_cache, band: 2, factor: 0.1},
+           ]
+    """
+    def __init__(self, mode=None):
+        self.ops = []
+        self.cacheable = True
+        self.mode = mode
+        self.max_band = {}
+        self.max_src_images = 0
+
+    def add_ops(self, dst_band, src_img, src_band, factor=1.0):
+        self.ops.append(band_ops(
+            dst_band=dst_band,
+            src_img=src_img,
+            src_band=src_band,
+            factor=factor,
+         ))
+        # store highest requested band index for each source
+        self.max_band[src_img] = max(self.max_band.get(src_img, 0), src_band)
+        self.max_src_images = max(src_img+1, self.max_src_images)
+
+    def merge(self, sources, image_opts, size=None, bbox=None, bbox_srs=None, coverage=None):
+        if len(sources) < self.max_src_images:
+            return BlankImageSource(size=size, image_opts=image_opts, cacheable=True)
+
+        if size is None:
+            size = sources[0].size
+
+        # load src bands
+        src_img_bands = []
+        for i, layer_img in enumerate(sources):
+            img = layer_img.as_image()
+
+            if i not in self.max_band:
+                # do not split img if not requested by any op
+                src_img_bands.append(None)
+                continue
+
+            if self.max_band[i] == 3 and img.mode != 'RGBA':
+                # convert to RGBA if band idx 3 is requestd (e.g. P or RGB src)
+                img = img.convert('RGBA')
+            elif img.mode == 'P':
+                img = img.convert('RGB')
+            src_img_bands.append(img.split())
+
+        tmp_mode = self.mode
+
+        if tmp_mode == 'RGBA':
+            result_bands = [None, None, None, None]
+        elif tmp_mode == 'RGB':
+            result_bands = [None, None, None]
+        elif tmp_mode == 'L':
+            result_bands = [None]
+        else:
+            raise ValueError("unsupported destination mode %s", image_opts.mode)
+
+        for op in self.ops:
+            chan = src_img_bands[op.src_img][op.src_band]
+            if op.factor != 1.0:
+                chan = ImageMath.eval("convert(int(float(a) * %f), 'L')" % op.factor, a=chan)
+                if result_bands[op.dst_band] is None:
+                    result_bands[op.dst_band] = chan
+                else:
+                    result_bands[op.dst_band] = ImageChops.add(
+                        result_bands[op.dst_band],
+                        chan,
+                    )
+            else:
+                result_bands[op.dst_band] = chan
+
+        for i, b in enumerate(result_bands):
+            if b is None:
+                # band not set
+                b = Image.new("L", size, 255 if i == 3 else 0)
+                result_bands[i] = b
+
+        result = Image.merge(tmp_mode, result_bands)
+        return ImageSource(result, size=size, image_opts=image_opts)
+
+
+def merge_images(layers, image_opts, size=None, bbox=None, bbox_srs=None, merger=None):
     """
     Merge multiple images into one.
 
@@ -132,12 +236,27 @@ def merge_images(images, image_opts, size=None):
     :param format: the format of the output `ImageSource`
     :param size: size of the merged image, if ``None`` the size
                  of the first image is used
+    :param bbox: Bounding box
+    :param bbox_srs: Bounding box SRS
+    :param merger: Image merger
     :rtype: `ImageSource`
     """
-    merger = LayerMerger()
-    for img in images:
-        merger.add(img)
-    return merger.merge(image_opts=image_opts, size=size)
+    if merger is None:
+        merger = LayerMerger()
+
+    # BandMerger does not have coverage support, passing only images
+    if isinstance(merger, BandMerger):
+        sources = [l[0] if isinstance(l, tuple) else l for l in layers]
+        return merger.merge(sources, image_opts=image_opts, size=size, bbox=bbox, bbox_srs=bbox_srs)
+
+    for layer in layers:
+        if isinstance(layer, tuple):
+            merger.add(layer[0], layer[1])
+        else:
+            merger.add(layer)
+
+    return merger.merge(image_opts=image_opts, size=size, bbox=bbox, bbox_srs=bbox_srs)
+
 
 def concat_legends(legends, format='png', size=None, bgcolor='#ffffff', transparent=True):
     """
@@ -181,3 +300,17 @@ def concat_legends(legends, format='png', size=None, bgcolor='#ffffff', transpar
         else:
             img.paste(legend_img, (0, legend_position_y[i]))
     return ImageSource(img, image_opts=ImageOptions(format=format))
+
+def concat_json_legends(legends):
+    """
+    Merge multiple json legends into one
+    :param legends: list of HttpResponse objects
+    :rtype json formatted string
+    """
+    legends_dict = {'Legend': []}
+    legends = legends[:]
+    for legend in legends:
+        legend_str = legend.read().decode('utf-8')
+        legend_dict = json.loads(legend_str)
+        legends_dict['Legend'].append(legend_dict['Legend'][0])
+    return json.dumps(legends_dict)
